@@ -1,6 +1,6 @@
-# mod_11_10.py - functions for performing the ISO 7064 Mod 11, 10 algorithm
+# mod_11_2.py - functions for performing the ISO 7064 Mod 11, 2 algorithm
 #
-# Copyright (C) 2010, 2011, 2012 Arthur de Jong
+# Copyright (C) 2010, 2011, 2012, 2013 Arthur de Jong
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -17,42 +17,58 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 # 02110-1301 USA
 
-"""The ISO 7064 Mod 11, 10 algorithm.
+"""The ISO 7064 Mod 11, 2 algorithm.
 
-The Mod 11, 10 algorithm uses a number of calculations modulo 11 and 10 to
-determine a checksum.
+The Mod 11, 2 algorithm is a simple module 11 checksum where the check
+digit can be an X to make the number valid.
 
-For a module that can do generic Mod x+1, x calculations see the
-:mod:`stdnum.iso7064.mod_37_36` module.
+For a module that can do generic Mod x, 2 calculations see the
+:mod:`stdnum.iso7064.mod_37_2` module.
 
->>> calc_check_digit('79462')
-'3'
->>> is_valid('794623')
-True
->>> calc_check_digit('00200667308')
-'5'
->>> is_valid('002006673085')
-True
+>>> calc_check_digit('0794')
+'0'
+>>> validate('07940')
+'07940'
+>>> calc_check_digit('079')
+'X'
+>>> validate('079X')
+'079X'
+>>> checksum('079X')
+1
 """
 
+from stdnum.exceptions import *
+
 
 def checksum(number):
     """Calculate the checksum. A valid number should have a checksum of 1."""
-    check = 5
+    check = 0
     for n in number:
-        check = (((check or 10) * 2) % 11 + int(n)) % 10
+        check = (2 * check + int(10 if n == 'X' else n)) % 11
     return check
 
 
 def calc_check_digit(number):
     """With the provided number, calculate the extra digit that should be
     appended to make it a valid number."""
-    return str((1 - ((checksum(number) or 10) * 2) % 11) % 10)
+    c = (1 - 2 * checksum(number)) % 11
+    return 'X' if c == 10 else str(c)
+
+
+def validate(number):
+    """Checks whether the check digit is valid."""
+    try:
+        valid = checksum(number) == 1
+    except:
+        raise InvalidFormat()
+    if not valid:
+        raise InvalidChecksum()
+    return number
 
 
 def is_valid(number):
     """Checks whether the check digit is valid."""
     try:
-        return bool(number) and checksum(number) == 1
-    except:
+        return bool(validate(number))
+    except ValidationError:
         return False
