@@ -1,6 +1,6 @@
 # util.py - common utility functions
 #
-# Copyright (C) 2012 Arthur de Jong
+# Copyright (C) 2012, 2013 Arthur de Jong
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -25,6 +25,14 @@ stdnum.
 """
 
 import pkgutil
+import pydoc
+import re
+import sys
+
+from stdnum.exceptions import *
+
+
+_strip_doctest_re = re.compile('^>>> .*\Z', re.DOTALL | re.MULTILINE)
 
 
 def clean(number, deletechars):
@@ -33,16 +41,41 @@ def clean(number, deletechars):
     >>> clean('123-456:78 9', ' -:')
     '123456789'
     """
-    return ''.join(x for x in number if x not in deletechars)
+    try:
+        return ''.join(x for x in number if x not in deletechars)
+    except:
+        raise InvalidFormat()
 
 
 def get_number_modules(base='stdnum'):
     """Yield all the module and package names under the specified module."""
-    module = __import__(base, globals(), locals(), [base])
+    __import__(base)
+    module = sys.modules[base]
     for loader, name, is_pkg in pkgutil.walk_packages(
                     module.__path__, module.__name__ + '.',
                     onerror=lambda x: None
                 ):
-        module = __import__(name, globals(), locals(), [name])
-        if hasattr(module, 'is_valid'):
+        __import__(name)
+        module = sys.modules[name]
+        if hasattr(module, 'validate'):
             yield module
+
+
+def get_module_name(module):
+    """Return the short description of the number."""
+    return pydoc.splitdoc(pydoc.getdoc(module))[0].strip('.')
+
+
+def get_module_description(module):
+    """Return a description of the number."""
+    doc = pydoc.splitdoc(pydoc.getdoc(module))[1]
+    # remove the doctests
+    return _strip_doctest_re.sub('', doc[1]).strip(),
+
+
+def get_module_list():
+    for module in get_number_modules():
+        yield ' * %s: %s' % (
+            module.__name__.replace('stdnum.', ''),
+            get_module_name(module),
+        )
